==> 1.8.10.tar.gz: lib/FFIType.dart <==
import 'dart:convert';
import 'dart:ffi';
import 'dart:typed_data';

import 'package:ffi/ffi.dart';
import 'package:unit_rc/UnitRCLog.dart';
import 'package:unit_rc/internal_type/InternalClassListCpp.dart';
import 'package:unit_rc/internal_type/InternalClassListDart.dart';

import 'UnitRCDartRuntime.dart';

Pointer<Uint8> createUint8PointerFromUint8List(List<int> data) {
  final blob = calloc<Uint8>(data.length);
  final blobBytes = blob.asTypedList(data.length);
  blobBytes.setAll(0, data);
  return blob;
}

Pointer<Int64> createInt64PointerFromInt8List(List<int> data) {
  final blob = calloc<Int64>(data.length);
  final blobBytes = blob.asTypedList(data.length);
  blobBytes.setAll(0, data);
  return blob;
}

StringFFI createUint8PointerFromString(String data) {
  final units = utf8.encode(data);
  final Pointer<Uint8> result = malloc<Uint8>(units.length + 1);
  final Uint8List nativeString = result.asTypedList(units.length + 1);
  nativeString.setAll(0, units);
  nativeString[units.length] = 0;
  return StringFFI(result, units.length);
}

class FFIType {
  static void freeCppUint8ListFFI(Pointer<Uint8ListFFI> pointer) {
    _native_FreeCppUint8ListFFI(pointer);
  }

  static final _native_FreeCppUint8ListFFI = unitRCLib.lookupFunction<
      Void Function(Pointer<Uint8ListFFI>),
      void Function(Pointer<Uint8ListFFI>)>("UnitRC_FreeUint8ListFFI");

  static void freeCppUint8ListArrayFFI(Pointer<Uint8ListArrayFFI> pointer) {
    _native_FreeCppUint8ListArrayFFI(pointer);
  }

  static final _native_FreeCppUint8ListArrayFFI = unitRCLib.lookupFunction<
      Void Function(Pointer<Uint8ListArrayFFI>),
      void Function(Pointer<Uint8ListArrayFFI>)>("UnitRC_FreeUint8ListArrayFFI");

  static void freeCppInt64ArrayFFI(Pointer<Int64ArrayFFI> pointer) {
    _native_FreeCppInt64ArrayFFI(pointer);
  }

  static final _native_FreeCppInt64ArrayFFI = unitRCLib.lookupFunction<
      Void Function(Pointer<Int64ArrayFFI>),
      void Function(Pointer<Int64ArrayFFI>)>("UnitRC_FreeInt64ArrayFFI");

  static void freeUint8ListFFI(Pointer<Uint8ListFFI> pointer) {
    Uint8ListFFI.free(pointer);
  }
  static final freeUint8ListPointerFP =
  Pointer.fromFunction<Void Function(Pointer<Uint8ListFFI>)>(freeUint8ListFFI);
  static final registerFreeUint8ListPointerFP = unitRCLib.lookupFunction<
      Void Function(Pointer<NativeFunction<Void Function(Pointer<Uint8ListFFI>)>>),
      void Function(Pointer<NativeFunction<Void Function(Pointer<Uint8ListFFI>)>>)
  >("UnitRC_Register_DartFreeUint8ListFFI");

  static void freeUint8ListArrayFFI(Pointer<Uint8ListArrayFFI> pointer) {
    Uint8ListArrayFFI.free(pointer);
  }
  static final freeUint8ListArrayPointerFP = Pointer.fromFunction<Void Function(Pointer<Uint8ListArrayFFI>)>(freeUint8ListArrayFFI);
  static final registerFreeUint8ListArrayPointerFP = unitRCLib.lookupFunction<
      Void Function(Pointer<NativeFunction<Void Function(Pointer<Uint8ListArrayFFI>)>>),
      void Function(Pointer<NativeFunction<Void Function(Pointer<Uint8ListArrayFFI>)>>)
  >("UnitRC_Register_DartFreeUint8ListArrayFFI");

  static void freeInt64ArrayFFI(Pointer<Int64ArrayFFI> pointer) {
    Int64ArrayFFI.free(pointer);
  }
  static final freeInt64ArrayPointerFP = Pointer.fromFunction<Void Function(Pointer<Int64ArrayFFI>)>(freeInt64ArrayFFI);
  static final registerFreeInt64ArrayPointerFP = unitRCLib.lookupFunction<
      Void Function(Pointer<NativeFunction<Void Function(Pointer<Int64ArrayFFI>)>>),
      void Function(Pointer<NativeFunction<Void Function(Pointer<Int64ArrayFFI>)>>)>("UnitRC_Register_DartFreeInt64ArrayFFI");

  static bool _isRegistered = false;
  static void registerNativeMethods() {
    if (_isRegistered) return;
    _isRegistered = true;
    UnitRCLog.info("registerNativeMethods FFIType start");
    registerFreeUint8ListPointerFP(freeUint8ListPointerFP);
    registerFreeUint8ListArrayPointerFP(freeUint8ListArrayPointerFP);
    registerFreeInt64ArrayPointerFP(freeInt64ArrayPointerFP);
    InternalClassListCpp.registerNativeMethods();
    InternalClassListDart.registerNativeMethods();
    UnitRCLog.info("registerNativeMethods FFIType end");
  }
}

class Int64ArrayFFI extends Struct {
  external Pointer<Int64> array;
  @Int64()
  external int length;

  List<int> convertToIntList() {
    return array.asTypedList(length);
  }

  static Pointer<Int64ArrayFFI> create(List<int> data) {
    final Pointer<Int64ArrayFFI> pointer = calloc.call(sizeOf<Int64ArrayFFI>());
    pointer.ref.array = createInt64PointerFromInt8List(data);
    pointer.ref.length = data.length;
    return pointer;
  }

  static free(Pointer<Int64ArrayFFI> pointer) {
    calloc.free(pointer.ref.array);
    calloc.free(pointer);
  }
}

class Uint8ListFFI extends Struct {
  external Pointer<Int8> str;

  @Int64()
  external int length;

  String convertToString() {
    return str.cast<Utf8>().toDartString(length: length);
  }

  Uint8List convertToUint8List() {
    return Uint8List.fromList(str.cast<Uint8>().asTypedList(length));
  }

  static Pointer<Uint8ListFFI> create(Uint8List data) {
    return createWithInt8Pinter(
        createUint8PointerFromUint8List(data).cast(), data.length);
  }

  static Pointer<Uint8ListFFI> createWithInt8Pinter(
      Pointer<Int8> str, int length) {
    final Pointer<Uint8ListFFI> pointer = calloc.call(sizeOf<Uint8ListFFI>());
    pointer.ref.str = str;
    pointer.ref.length = length;
    return pointer;
  }

  static Pointer<Uint8ListFFI> createWithString(String data) {
    final ffi = createUint8PointerFromString(data);
    return createWithInt8Pinter(ffi.str.cast(), ffi.length);
  }

  static free(Pointer<Uint8ListFFI> pointer) {
    calloc.free(pointer.ref.str);
    calloc.free(pointer);
  }
}

class Uint8ListArrayFFI extends Struct {
  external Pointer<Uint8ListFFI> array;

  @Int64()
  external int length;

  List<String> convertToStringList() {
    List<String> result = [];
    for (var i = 0; i < length; i++) {
      final itemStruct = array.elementAt(i).ref;
      String item = itemStruct.convertToString();
      result.add(item);
    }
    return result;
  }

  List<Uint8List> convertToUint8ListList() {
    List<Uint8List> result = [];
    for (var i = 0; i < length; i++) {
      final itemStruct = array.elementAt(i).ref;
      Uint8List item = itemStruct.convertToUint8List();
      result.add(item);
    }
    return result;
  }

  static Pointer<Uint8ListArrayFFI> create(List<Uint8List> data) {
    final sizeOfStruct = sizeOf<Uint8ListFFI>();
    final Pointer<Uint8ListFFI> nativeUint8ListBuffer =
        calloc.allocate(sizeOfStruct * (data.length));
    for (int i = 0; i < data.length; i++) {
      nativeUint8ListBuffer.elementAt(i).ref.str =
          createUint8PointerFromUint8List(data[i]).cast<Int8>();
      nativeUint8ListBuffer.elementAt(i).ref.length = data[i].length;
    }

    return Uint8ListArrayFFI.createWithUint8ListFFIPointer(
        nativeUint8ListBuffer, data.length);
  }

  static Pointer<Uint8ListArrayFFI> createWithStringList(List<String> data) {
    final sizeOfStruct = sizeOf<Uint8ListFFI>();
    final Pointer<Uint8ListFFI> nativeUint8ListBuffer =
        calloc.allocate(sizeOfStruct * (data.length));
    for (int i = 0; i < data.length; i++) {
      final ffi = createUint8PointerFromString(data[i]);
      nativeUint8ListBuffer.elementAt(i).ref.str = ffi.str.cast();
      nativeUint8ListBuffer.elementAt(i).ref.length = ffi.length;
    }

    return Uint8ListArrayFFI.createWithUint8ListFFIPointer(
        nativeUint8ListBuffer, data.length);
  }

  static Pointer<Uint8ListArrayFFI> createWithUint8ListFFIPointer(
      Pointer<Uint8ListFFI> array, int length) {
    final Pointer<Uint8ListArrayFFI> pointer =
        calloc.allocate<Uint8ListArrayFFI>(sizeOf<Uint8ListArrayFFI>());
    pointer.ref.array = array;
    pointer.ref.length = length;
    return pointer;
  }

  static free(Pointer<Uint8ListArrayFFI> pointer) {
    for (int i = 0; i < pointer.ref.length; i++) {
      final ref = pointer.ref.array.elementAt(i).ref;
      calloc.free(ref.str);
    }
    calloc.free(pointer.ref.array);
    calloc.free(pointer);
  }
}

class StringFFI {
  final Pointer<Uint8> str;
  final int length;

  StringFFI(this.str, this.length);
}

==> 1.8.19.tar.gz: lib/FFIType.dart <==
import 'dart:convert';
import 'dart:ffi';
import 'dart:typed_data';

import 'package:ffi/ffi.dart';
import 'package:unit_rc/UnitRCLog.dart';
import 'package:unit_rc/internal_type/InternalClassListDart.dart';
import 'package:unit_rc/ur/InternalClassListCppBridge.dart';
import 'package:unit_rc/ur/InternalClassListDartBridge.dart';

import 'UnitRCDartRuntime.dart';

Pointer<Uint8> createUint8PointerFromUint8List(List<int> data) {
  final blob = calloc<Uint8>(data.length);
  final blobBytes = blob.asTypedList(data.length);
  blobBytes.setAll(0, data);
  return blob;
}

Pointer<Int64> createInt64PointerFromInt8List(List<int> data) {
  final blob = calloc<Int64>(data.length);
  final blobBytes = blob.asTypedList(data.length);
  blobBytes.setAll(0, data);
  return blob;
}

StringFFI createUint8PointerFromString(String data) {
  final units = utf8.encode(data);
  final Pointer<Uint8> result = malloc<Uint8>(units.length + 1);
  final Uint8List nativeString = result.asTypedList(units.length + 1);
  nativeString.setAll(0, units);
  nativeString[units.length] = 0;
  return StringFFI(result, units.length);
}

class FFIType {
  static void freeCppUint8ListFFI(Pointer<Uint8ListFFI> pointer) {
    _native_FreeCppUint8ListFFI(pointer);
  }

  static final _native_FreeCppUint8ListFFI = unitRCLib.lookupFunction<
      Void Function(Pointer<Uint8ListFFI>),
      void Function(Pointer<Uint8ListFFI>)>("UnitRC_FreeUint8ListFFI");

  static void freeCppUint8ListArrayFFI(Pointer<Uint8ListArrayFFI> pointer) {
    _native_FreeCppUint8ListArrayFFI(pointer);
  }

  static final _native_FreeCppUint8ListArrayFFI = unitRCLib.lookupFunction<
      Void Function(Pointer<Uint8ListArrayFFI>),
      void Function(Pointer<Uint8ListArrayFFI>)>("UnitRC_FreeUint8ListArrayFFI");

  static void freeCppInt64ArrayFFI(Pointer<Int64ArrayFFI> pointer) {
    _native_FreeCppInt64ArrayFFI(pointer);
  }

  static final _native_FreeCppInt64ArrayFFI = unitRCLib.lookupFunction<
      Void Function(Pointer<Int64ArrayFFI>),
      void Function(Pointer<Int64ArrayFFI>)>("UnitRC_FreeInt64ArrayFFI");

  static void freeUint8ListFFI(Pointer<Uint8ListFFI> pointer) {
    Uint8ListFFI.free(pointer);
  }
  static final freeUint8ListPointerFP =
  Pointer.fromFunction<Void Function(Pointer<Uint8ListFFI>)>(freeUint8ListFFI);
  static final registerFreeUint8ListPointerFP = unitRCLib.lookupFunction<
      Void Function(Pointer<NativeFunction<Void Function(Pointer<Uint8ListFFI>)>>),
      void Function(Pointer<NativeFunction<Void Function(Pointer<Uint8ListFFI>)>>)
  >("UnitRC_Register_DartFreeUint8ListFFI");

  static void freeUint8ListArrayFFI(Pointer<Uint8ListArrayFFI> pointer) {
    Uint8ListArrayFFI.free(pointer);
  }
  static final freeUint8ListArrayPointerFP = Pointer.fromFunction<Void Function(Pointer<Uint8ListArrayFFI>)>(freeUint8ListArrayFFI);
  static final registerFreeUint8ListArrayPointerFP = unitRCLib.lookupFunction<
      Void Function(Pointer<NativeFunction<Void Function(Pointer<Uint8ListArrayFFI>)>>),
      void Function(Pointer<NativeFunction<Void Function(Pointer<Uint8ListArrayFFI>)>>)
  >("UnitRC_Register_DartFreeUint8ListArrayFFI");

  static void freeInt64ArrayFFI(Pointer<Int64ArrayFFI> pointer) {
    Int64ArrayFFI.free(pointer);
  }
  static final freeInt64ArrayPointerFP = Pointer.fromFunction<Void Function(Pointer<Int64ArrayFFI>)>(freeInt64ArrayFFI);
  static final registerFreeInt64ArrayPointerFP = unitRCLib.lookupFunction<
      Void Function(Pointer<NativeFunction<Void Function(Pointer<Int64ArrayFFI>)>>),
      void Function(Pointer<NativeFunction<Void Function(Pointer<Int64ArrayFFI>)>>)>("UnitRC_Register_DartFreeInt64ArrayFFI");

  static bool _isRegistered = false;
  static void registerNativeMethods() {
    if (_isRegistered) return;
    _isRegistered = true;
    UnitRCLog.info("registerNativeMethods FFIType start");
    registerFreeUint8ListPointerFP(freeUint8ListPointerFP);
    registerFreeUint8ListArrayPointerFP(freeUint8ListArrayPointerFP);
    registerFreeInt64ArrayPointerFP(freeInt64ArrayPointerFP);
    InternalClassListCppBridge.registerNativeMethods();
    InternalClassListDartBridge.registerNativeMethods();
    InternalClassListDart.registerNativeMethods();
    UnitRCLog.info("registerNativeMethods FFIType end");
  }
}

class Int64ArrayFFI extends Struct {
  external Pointer<Int64> array;
  @Int64()
  external int length;

  List<int> convertToIntList() {
    return array.asTypedList(length);
  }

  static Pointer<Int64ArrayFFI> create(List<int> data) {
    final Pointer<Int64ArrayFFI> pointer = calloc.call(sizeOf<Int64ArrayFFI>());
    pointer.ref.array = createInt64PointerFromInt8List(data);
    pointer.ref.length = data.length;
    return pointer;
  }

  static free(Pointer<Int64ArrayFFI> pointer) {
    calloc.free(pointer.ref.array);
    calloc.free(pointer);
  }
}

class Uint8ListFFI extends Struct {
  external Pointer<Int8> str;

  @Int64()
  external int length;

  String convertToString() {
    return str.cast<Utf8>().toDartString(length: length);
  }

  Uint8List convertToUint8List() {
    return Uint8List.fromList(str.cast<Uint8>().asTypedList(length));
  }

  static Pointer<Uint8ListFFI> create(Uint8List data) {
    return createWithInt8Pinter(
        createUint8PointerFromUint8List(data).cast(), data.length);
  }

  static Pointer<Uint8ListFFI> createWithInt8Pinter(
      Pointer<Int8> str, int length) {
    final Pointer<Uint8ListFFI> pointer = calloc.call(sizeOf<Uint8ListFFI>());
    pointer.ref.str = str;
    pointer.ref.length = length;
    return pointer;
  }

  static Pointer<Uint8ListFFI> createWithString(String data) {
    final ffi = createUint8PointerFromString(data);
    return createWithInt8Pinter(ffi.str.cast(), ffi.length);
  }

  static free(Pointer<Uint8ListFFI> pointer) {
    calloc.free(pointer.ref.str);
    calloc.free(pointer);
  }
}

class Uint8ListArrayFFI extends Struct {
  external Pointer<Uint8ListFFI> array;

  @Int64()
  external int length;

  List<String> convertToStringList() {
    List<String> result = [];
    for (var i = 0; i < length; i++) {
      final itemStruct = array.elementAt(i).ref;
      String item = itemStruct.convertToString();
      result.add(item);
    }
    return result;
  }

  List<Uint8List> convertToUint8ListList() {
    List<Uint8List> result = [];
    for (var i = 0; i < length; i++) {
      final itemStruct = array.elementAt(i).ref;
      Uint8List item = itemStruct.convertToUint8List();
      result.add(item);
    }
    return result;
  }

  static Pointer<Uint8ListArrayFFI> create(List<Uint8List> data) {
    final sizeOfStruct = sizeOf<Uint8ListFFI>();
    final Pointer<Uint8ListFFI> nativeUint8ListBuffer =
        calloc.allocate(sizeOfStruct * (data.length));
    for (int i = 0; i < data.length; i++) {
      nativeUint8ListBuffer.elementAt(i).ref.str =
          createUint8PointerFromUint8List(data[i]).cast<Int8>();
      nativeUint8ListBuffer.elementAt(i).ref.length = data[i].length;
    }

    return Uint8ListArrayFFI.createWithUint8ListFFIPointer(
        nativeUint8ListBuffer, data.length);
  }

  static Pointer<Uint8ListArrayFFI> createWithStringList(List<String> data) {
    final sizeOfStruct = sizeOf<Uint8ListFFI>();
    final Pointer<Uint8ListFFI> nativeUint8ListBuffer =
        calloc.allocate(sizeOfStruct * (data.length));
    for (int i = 0; i < data.length; i++) {
      final ffi = createUint8PointerFromString(data[i]);
      nativeUint8ListBuffer.elementAt(i).ref.str = ffi.str.cast();
      nativeUint8ListBuffer.elementAt(i).ref.length = ffi.length;
    }

    return Uint8ListArrayFFI.createWithUint8ListFFIPointer(
        nativeUint8ListBuffer, data.length);
  }

  static Pointer<Uint8ListArrayFFI> createWithUint8ListFFIPointer(
      Pointer<Uint8ListFFI> array, int length) {
    final Pointer<Uint8ListArrayFFI> pointer =
        calloc.allocate<Uint8ListArrayFFI>(sizeOf<Uint8ListArrayFFI>());
    pointer.ref.array = array;
    pointer.ref.length = length;
    return pointer;
  }

  static free(Pointer<Uint8ListArrayFFI> pointer) {
    for (int i = 0; i < pointer.ref.length; i++) {
      final ref = pointer.ref.array.elementAt(i).ref;
      calloc.free(ref.str);
    }
    calloc.free(pointer.ref.array);
    calloc.free(pointer);
  }
}

class StringFFI {
  final Pointer<Uint8> str;
  final int length;

  StringFFI(this.str, this.length);
}
